﻿Imports System.Threading
Imports System.Threading.Tasks
Imports [Global].Fakes
Imports Microsoft.VisualStudio.TestTools.UnitTesting

<TestClass>
Public Class MSStubTests
	<TestMethod>
	Public async Function RetrieveValue_SynchronousSuccess_Adds42() As Task
		Dim stub = New StubIMyService With {.GetAsync = Function() Task.FromResult(5)}
		Dim system = New SystemUnderTest(stub)

        Dim result = Await system.RetrieveValueAsync

		Assert.AreEqual(47, result)
	End Function

	<TestMethod>
	Public async Function RetrieveValue_AsynchronousSuccess_Adds42() As Task
        Dim service = New StubIMyService With {.GetAsync = Async Function()
                                                               Await Task.Yield
                                                               Return 5
                                                           End Function}
        Dim system = New SystemUnderTest(service)

        Dim result = Await system.RetrieveValueAsync

        Assert.AreEqual(47, result)
	End Function

	<TestMethod>
	Public async Function RetrieveValue_AsynchronousFailure_Adds42() As Task
        Dim service = New StubIMyService With {.GetAsync = Async Function()
                                                               Await Task.Yield
                                                               Throw New Exception
                                                           End Function}
        Dim system = New SystemUnderTest(service)

        Await AssertEx.ThrowsAsync(Function() system.RetrieveValueAsync)
	End Function

	<TestMethod>
	Public async Function Stubs_NotAsNiceDefaultsForAsyncMethods() As Task
        Dim service = New StubIMyService
		service.BehaveAsDefaultValue()
		Dim system = New SystemUnderTest(service)

        Dim result = Await system.RetrieveValueAsync

		Assert.AreEqual(42, result)
	End Function
End Class