﻿Imports System.Threading.Tasks
Imports Microsoft.VisualStudio.TestPlatform.UnitTestFramework
Imports System

Namespace AssertExUnitTests
    <TestClass>
    Public Class AssertExWinStoreMSUnitTests
        <TestMethod>
        Public Async Function NoException_Synchronous_Fails() As Task
            Try
                Await AssertEx.ThrowsAsync(Function() Task.FromResult(0))
                Assert.Fail()
            Catch ex As Exception
                Assert.AreEqual("Delegate did not throw expected exception Exception.", ex.Message)
            End Try
        End Function

        <TestMethod>
        Public Async Function NoException_Asynchronous_Fails() As Task
            Try
                Await AssertEx.ThrowsAsync(Function() Task.Delay(100))
                Assert.Fail()
            Catch ex As Exception
                Assert.AreEqual("Delegate did not throw expected exception Exception.", ex.Message)
            End Try
        End Function

        <TestMethod>
        Public Async Function ExpectedException_Synchronous_Passes() As Task
            Dim thrown As InvalidOperationException = New InvalidOperationException
            Dim result As Exception = Await AssertEx.ThrowsAsync(Of InvalidOperationException)(Function()
                                                                                                   Throw thrown
                                                                                               End Function)
            Assert.AreSame(thrown, result)
        End Function

        <TestMethod>
        Public Async Function ExpectedException_Asynchronous_Passes() As Task
            Dim thrown As Exception = New Exception
            Dim result As Exception = Await AssertEx.ThrowsAsync(Async Function()
                                                                     Await Task.Delay(100)
                                                                     Throw thrown
                                                                 End Function)
            Assert.AreSame(thrown, result)
        End Function

        <TestMethod>
        Public Async Function ExpectedDerivedException_Synchronous_Passes() As Task
            Dim thrown As InvalidOperationException = New InvalidOperationException
            Dim result As Exception = Await AssertEx.ThrowsAsync(Of Exception)(Function()
                                                                                   Throw thrown
                                                                               End Function)
            Assert.AreSame(thrown, result)
        End Function

        <TestMethod>
        Public Async Function ExpectedDerivedException_Asynchronous_Passes() As Task
            Dim thrown As Exception = New InvalidOperationException
            Dim result As Exception = Await AssertEx.ThrowsAsync(Of Exception)(Async Function()
                                                                                   Await Task.Delay(100)
                                                                                   Throw thrown
                                                                               End Function)
            Assert.AreSame(thrown, result)
        End Function

        <TestMethod>
        Public Async Function UnexpectedDerivedException_Synchronous_Fails() As Task
            Try
                Dim thrown As Exception = New InvalidOperationException
                Dim result As Exception = Await AssertEx.ThrowsAsync(Of Exception)(Function()
                                                                                       Throw thrown
                                                                                   End Function, False)
                Assert.Fail()
            Catch ex As Exception
                Assert.AreEqual("Delegate threw exception of type InvalidOperationException, but Exception was expected.", ex.Message)
            End Try
        End Function

        <TestMethod>
        Public Async Function UnexpectedDerivedException_Asynchronous_Fails() As Task
            Try
                Dim thrown As Exception = New InvalidOperationException
                Dim result As Exception = Await AssertEx.ThrowsAsync(Of Exception)(Async Function()
                                                                                       Await Task.Delay(100)
                                                                                       Throw thrown
                                                                                   End Function, False)
                Assert.Fail()
            Catch ex As Exception
                Assert.AreEqual("Delegate threw exception of type InvalidOperationException, but Exception was expected.", ex.Message)
            End Try
        End Function

        <TestMethod>
        Public Async Function UnexpectedException_Synchronous_Fails() As Task
            Try
                Dim thrown As Exception = New InvalidOperationException
                Dim result As Exception = Await AssertEx.ThrowsAsync(Of ArgumentException)(Function()
                                                                                               Throw thrown
                                                                                           End Function)
                Assert.Fail()
            Catch ex As Exception
                Assert.AreEqual("Delegate threw exception of type InvalidOperationException, but ArgumentException or a derived type was expected.", ex.Message)
            End Try
        End Function

        <TestMethod>
        Public Async Function UnexpectedException_Asynchronous_Fails() As Task
            Try
                Dim thrown As Exception = New InvalidOperationException
                Dim result As Exception = Await AssertEx.ThrowsAsync(Of ArgumentException)(Async Function()
                                                                                               Await Task.Delay(100)
                                                                                               Throw thrown
                                                                                           End Function)
                Assert.Fail()
            Catch ex As Exception
                Assert.AreEqual("Delegate threw exception of type InvalidOperationException, but ArgumentException or a derived type was expected.", ex.Message)
            End Try
        End Function
    End Class
End Namespace
