﻿Imports System.Threading.Tasks
Imports Microsoft.VisualStudio.TestTools.UnitTesting

Namespace AssertExUnitTests
	<TestClass>
	Public Class AssertExMSUnitTests
		<TestMethod>
		Public async Function NoException_Synchronous_Fails() As Task
			Try
				await AssertEx.ThrowsAsync(Function() Task.FromResult(0))
				Assert.Fail()
			Catch ex As Exception
				Assert.AreEqual("Delegate did not throw expected exception Exception.", ex.Message)
			End Try
		End Function

		<TestMethod>
		Public async Function NoException_Asynchronous_Fails() As Task
			Try
				await AssertEx.ThrowsAsync(Function() Task.Delay(100))
				Assert.Fail()
			Catch ex As Exception
				Assert.AreEqual("Delegate did not throw expected exception Exception.", ex.Message)
			End Try
		End Function

        <TestMethod>
        Public Async Function ExpectedException_Synchronous_Passes() As Task
            Dim thrown = New Exception
            Dim result = Await AssertEx.ThrowsAsync(Function()
                                                        Throw thrown
                                                    End Function)
            Assert.AreSame(thrown, result)
        End Function

		<TestMethod>
		Public async Function ExpectedException_Asynchronous_Passes() As Task
            Dim thrown = New Exception
            Dim result = Await AssertEx.ThrowsAsync(Async Function()
                                                                 Await Task.Delay(100)
                                                                 Throw thrown
                                                             End Function)
			Assert.AreSame(thrown, result)
		End Function

		<TestMethod>
		Public async Function ExpectedDerivedException_Synchronous_Passes() As Task
            Dim thrown = New InvalidOperationException
            Dim result = Await AssertEx.ThrowsAsync(Of Exception)(Function()
                                                                      Throw thrown
                                                                  End Function)
			Assert.AreSame(thrown, result)
		End Function

		<TestMethod>
		Public async Function ExpectedDerivedException_Asynchronous_Passes() As Task
            Dim thrown = New InvalidOperationException
            Dim result = Await AssertEx.ThrowsAsync(Of Exception)(Async Function()
                                                                               Await Task.Delay(100)
                                                                               Throw thrown
                                                                           End Function)
			Assert.AreSame(thrown, result)
		End Function

		<TestMethod>
		Public async Function UnexpectedDerivedException_Synchronous_Fails() As Task
			Try
                Dim thrown = New InvalidOperationException
                Dim result = Await AssertEx.ThrowsAsync(Of Exception)(Function()
                                                                          Throw thrown
                                                                      End Function, False)
				Assert.Fail()
			Catch ex As Exception
				Assert.AreEqual("Delegate threw exception of type InvalidOperationException, but Exception was expected.", ex.Message)
			End Try
		End Function

		<TestMethod>
		Public async Function UnexpectedDerivedException_Asynchronous_Fails() As Task
			Try
                Dim thrown = New InvalidOperationException
                Dim result = Await AssertEx.ThrowsAsync(Of Exception)(Async Function()
                                                                                      Await Task.Delay(100)
                                                                                      Throw thrown
                                                                                  End Function, False)
				Assert.Fail()
			Catch ex As Exception
				Assert.AreEqual("Delegate threw exception of type InvalidOperationException, but Exception was expected.", ex.Message)
			End Try
		End Function

		<TestMethod>
		Public async Function UnexpectedException_Synchronous_Fails() As Task
			Try
                Dim thrown = New InvalidOperationException
                Dim result = Await AssertEx.ThrowsAsync(Of ArgumentException)(Function()
                                                                                  Throw thrown
                                                                              End Function)
				Assert.Fail()
			Catch ex As Exception
				Assert.AreEqual("Delegate threw exception of type InvalidOperationException, but ArgumentException or a derived type was expected.", ex.Message)
			End Try
		End Function

		<TestMethod>
		Public async Function UnexpectedException_Asynchronous_Fails() As Task
			Try
                Dim thrown = New InvalidOperationException
                Dim result = Await AssertEx.ThrowsAsync(Of ArgumentException)(Async Function()
                                                                                              Await Task.Delay(100)
                                                                                              Throw thrown
                                                                                          End Function)
                Assert.Fail()
			Catch ex As Exception
				Assert.AreEqual("Delegate threw exception of type InvalidOperationException, but ArgumentException or a derived type was expected.", ex.Message)
			End Try
		End Function
	End Class
End Namespace
