﻿Imports System.Threading
Imports System.Threading.Tasks
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Moq

<TestClass>
Public Class MSUnitTests
	' Warning: bad code!
    <TestMethod>
    Public Sub IncorrectlyPassingTest()
        SystemUnderTest.FailAsync()
    End Sub

	<TestMethod>
	Public async Function CorrectlyFailingTest() As Task
        Await SystemUnderTest.FailAsync
	End Function

	' MSTest will refuse to execute this test (UTA007).
	<TestMethod>
	Public async Sub SchrodingerTest()
        Await SystemUnderTest.FailAsync
	End Sub

	' MSTest will refuse to execute this test (UTA007).
	<TestMethod>
	Public async Sub SynchronizationContextNotDefinedForAsyncVoid()
		Assert.IsNull(SynchronizationContext.Current)
		await Task.FromResult(0)
	End Sub

	<TestMethod>
	Public async Function SynchronizationContextNotDefinedForAsyncTask() As Task
		Assert.IsNull(SynchronizationContext.Current)
		await Task.FromResult(0)
	End Function

	<TestMethod>
	Public Sub SynchronizationContextNotDefinedForSynchronousTest()
		Assert.IsNull(SynchronizationContext.Current)
	End Sub

	' Old-style; only works on desktop.
	<TestMethod, ExpectedException(GetType(Exception))>
	Public Sub SynchronousFailureTest_ExpectedException()
		SystemUnderTest.Fail()
	End Sub

	<TestMethod, ExpectedException(GetType(Exception))>
	Public async Function FailureTest_ExpectedException() As Task
        Await SystemUnderTest.FailAsync
	End Function

	<TestMethod>
	Public async Function FailureTest_AssertEx() As Task
        Dim ex = Await AssertEx.ThrowsAsync(Function() SystemUnderTest.FailAsync)
		Assert.IsNotNull(ex)
	End Function

	<TestMethod>
	Public async Function RetrieveValue_SynchronousSuccess_Adds42() As Task
		Dim service = New Mock(Of IMyService)
		service.Setup(Function(x) x.GetAsync()).Returns(Function() Task.FromResult(5))
        ' Or: service.Setup(x => x.GetAsync()).ReturnsAsync(5)
		Dim system = New SystemUnderTest(service.Object)

        Dim result = Await system.RetrieveValueAsync

		Assert.AreEqual(47, result)
	End Function

	<TestMethod>
	Public async Function RetrieveValue_AsynchronousSuccess_Adds42() As Task
        Dim service = New Mock(Of IMyService)
        service.Setup(Function(x) x.GetAsync()).Returns(Async Function()
                                                                  Await Task.Yield
                                                                  Return 5
                                                              End Function)
		Dim system = New SystemUnderTest(service.Object)

        Dim result = Await system.RetrieveValueAsync

		Assert.AreEqual(47, result)
	End Function

	<TestMethod>
	Public async Function RetrieveValue_AsynchronousFailure_Adds42() As Task
        Dim service = New Mock(Of IMyService)
        service.Setup(Function(x) x.GetAsync()).Returns(Async Function()
                                                            Await Task.Yield
                                                            Throw New Exception
                                                        End Function)
		Dim system = New SystemUnderTest(service.Object)

        Await AssertEx.ThrowsAsync(Function() system.RetrieveValueAsync)
	End Function

	<TestMethod>
	Public async Function MoqHasNiceDefaultsForAsyncMethods() As Task
        Dim service = New Mock(Of IMyService)
		Dim system = New SystemUnderTest(service.Object)

        Dim result = Await system.RetrieveValueAsync

		Assert.AreEqual(42, result)
	End Function
End Class