﻿Imports System.Threading
Imports System.Threading.Tasks
Imports Moq
Imports NUnit.Framework

<TestFixture>
Public Class NUnit2Tests
	' Warning: bad code!
	<Test>
	Public Sub IncorrectlyPassingTest()
		SystemUnderTest.FailAsync()
	End Sub

	<Test>
	Public async Function CorrectlyFailingTest() As Task
        Await SystemUnderTest.FailAsync
	End Function

	' Correctly fails under NUnit 2.6.3, due to an injected NUnit.Framework.AsyncSynchronizationContext.
	' NOTE: NUnit 2.9.7 is expected to remove support for async void unit tests.
	<Test>
	Public async Sub SchrodingerTest()
        Await SystemUnderTest.FailAsync
	End Sub

	' NOTE: NUnit 2.9.7 is expected to remove support for async void unit tests.
	<Test>
	Public async Sub SynchronizationContextDefinedForAsyncVoid()
		Assert.IsNotNull(SynchronizationContext.Current)
		await Task.FromResult(0)
	End Sub

	<Test>
	Public async Function SynchronizationContextNotDefinedForAsyncTask() As Task
		Assert.IsNull(SynchronizationContext.Current)
		await Task.FromResult(0)
	End Function

	<Test>
	Public Sub SynchronizationContextNotDefinedForSynchronousTest()
		Assert.IsNull(SynchronizationContext.Current)
	End Sub

	<Test>
	Public Sub FailureTest_AssertThrows()
		' This works, though it actually implements a nested loop,
		'  synchronously blocking the Assert.Throws call until the asynchronous
		'  FailAsync call completes.
        Assert.Throws(Of Exception)(Async Sub()
                                        Await SystemUnderTest.FailAsync
                                    End Sub)
	End Sub

	' Does NOT pass.
	<Test>
	Public Sub BadFailureTest_AssertThrows()
        Assert.Throws(Of Exception)(Function() SystemUnderTest.FailAsync)
	End Sub

	<Test, ExpectedException>
	Public async Function FailureTest_ExpectedException() As Task
        Await SystemUnderTest.FailAsync
	End Function

    <Test>
    Public Sub FailureTest_AssertThat()
        ' This also works, though it actually implements a nested loop,
        '  synchronously blocking the Assert.That call until the asynchronous
        '  FailAsync call completes.
        Assert.That(Async Function()
                        Await SystemUnderTest.FailAsync
                    End Function, Throws.InstanceOf(Of Exception)())
    End Sub

	' Does NOT pass.
	<Test>
	Public Sub BadFailureTest_AssertThat()
        Assert.That(Function() SystemUnderTest.FailAsync, Throws.InstanceOf(Of Exception))
	End Sub

	<Test>
	Public async Function FailureTest_AssertEx() As Task
        Await AssertEx.ThrowsAsync(Function() SystemUnderTest.FailAsync)
	End Function

	<Test>
	Public async Function RetrieveValue_SynchronousSuccess_Adds42() As Task
        Dim service = New Mock(Of IMyService)
		service.Setup(Function(x) x.GetAsync()).Returns(Function() Task.FromResult(5))
        ' Or: service.Setup(x => x.GetAsync()).ReturnsAsync(5)
		Dim system = New SystemUnderTest(service.Object)

        Dim result = Await system.RetrieveValueAsync

		Assert.AreEqual(47, result)
	End Function

	<Test>
	Public async Function RetrieveValue_AsynchronousSuccess_Adds42() As Task
        Dim service = New Mock(Of IMyService)
        service.Setup(Function(x) x.GetAsync()).Returns(Async Function()
                                                                  Await Task.Yield
                                                                  Return 5
                                                              End Function)
		Dim system = New SystemUnderTest(service.Object)

        Dim result = Await system.RetrieveValueAsync

		Assert.AreEqual(47, result)
	End Function

	<Test>
	Public async Function RetrieveValue_AsynchronousFailure_Adds42() As Task
        Dim service = New Mock(Of IMyService)
        service.Setup(Function(x) x.GetAsync()).Returns(Async Function()
                                                            Await Task.Yield
                                                            Throw New Exception
                                                        End Function)
		Dim system = New SystemUnderTest(service.Object)

        Await AssertEx.ThrowsAsync(Function() system.RetrieveValueAsync)
	End Function

	<Test>
	Public async Function MoqHasNiceDefaultsForAsyncMethods() As Task
        Dim service = New Mock(Of IMyService)
		Dim system = New SystemUnderTest(service.Object)

        Dim result = Await system.RetrieveValueAsync

		Assert.AreEqual(42, result)
	End Function
End Class