﻿Imports System.Text
Imports System.Threading
Imports System.Threading.Tasks
Imports Moq
Imports Xunit

Public Class XUnit1Tests
	' Warning: bad code!
	<Fact>
	Public Sub IncorrectlyPassingTest()
		SystemUnderTest.FailAsync()
	End Sub

	<Fact>
	Public async Function CorrectlyFailingTest() As Task
        Await SystemUnderTest.FailAsync
	End Function

	<Fact>
	Public async Sub SchrodingerTest()
        Await SystemUnderTest.FailAsync
	End Sub

	<Fact>
	Public async Sub SynchronizationContextDefinedForAsyncVoid()
		Assert.Null(SynchronizationContext.Current)
		await Task.FromResult(0)
	End Sub

	<Fact>
	Public async Function SynchronizationContextNotDefinedForAsyncTask() As Task
		Assert.Null(SynchronizationContext.Current)
		await Task.FromResult(0)
	End Function

	<Fact>
	Public Sub SynchronizationContextNotDefinedForSynchronousTest()
		Assert.Null(SynchronizationContext.Current)
	End Sub

	' Does NOT pass.
	<Fact>
	Public Sub BadFailureTest_AssertThrows()
        Assert.Throws(Of Exception)(Async Sub()
                                        Await SystemUnderTest.FailAsync
                                    End Sub)
	End Sub

	<Fact>
	Public async Function FailureTest_AssertEx() As Task
        Await AssertEx.ThrowsAsync(Function() SystemUnderTest.FailAsync)
	End Function

	<Fact>
	Public async Function RetrieveValue_SynchronousSuccess_Adds42() As Task
        Dim service = New Mock(Of IMyService)
		service.Setup(Function(x) x.GetAsync()).Returns(Function() Task.FromResult(5))
        ' Or: service.Setup(x => x.GetAsync()).ReturnsAsync(5)
		Dim system = New SystemUnderTest(service.Object)

        Dim result = Await system.RetrieveValueAsync

		Assert.Equal(47, result)
	End Function

	<Fact>
	Public async Function RetrieveValue_AsynchronousSuccess_Adds42() As Task
        Dim service = New Mock(Of IMyService)
        service.Setup(Function(x) x.GetAsync()).Returns(Async Function()
                                                                  Await Task.Yield
                                                                  Return 5
                                                              End Function)
		Dim system = New SystemUnderTest(service.Object)

        Dim result = Await system.RetrieveValueAsync

		Assert.Equal(47, result)
	End Function

	<Fact>
	Public async Function RetrieveValue_AsynchronousFailure_Adds42() As Task
        Dim service = New Mock(Of IMyService)
        service.Setup(Function(x) x.GetAsync()).Returns(Async Function()
                                                            Await Task.Yield
                                                            Throw New Exception
                                                        End Function)
		Dim system = New SystemUnderTest(service.Object)

        Await AssertEx.ThrowsAsync(Function() system.RetrieveValueAsync)
	End Function

	<Fact>
	Public async Function MoqHasNiceDefaultsForAsyncMethods() As Task
        Dim service = New Mock(Of IMyService)
		Dim system = New SystemUnderTest(service.Object)

        Dim result = Await system.RetrieveValueAsync

		Assert.Equal(42, result)
	End Function
End Class
